#ifndef _Dialog_h
#define _Dialog_h

/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include <gtkmm/dialog.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/table.h>
#include <gtkmm/filechooserdialog.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/togglebutton.h>
#include <gtkmm/fontbutton.h>
#include <gtkmm/colorbutton.h>
#include <gtkmm/treeview.h>
#include <gtkmm/comboboxentrytext.h>
#include <gtkmm/comboboxtext.h>

#include <libglademm/xml.h>

#include "SubtitleTime.h"
#include "SubtitleModel.h"
#include "StyleModel.h"
#include "Document.h"
#include "SubtitleView.h"
#include <iostream>
#include "DialogFileChooser.h"

/*
 *
 */
class GtkSubtitleTime : public Gtk::Table
{
public:
  GtkSubtitleTime();

  void set_value(int h, int m, int s, int ms);
  void set_value(const SubtitleTime &time);

  SubtitleTime get_value();

protected:
  Gtk::Label      m_labelHours;
  Gtk::SpinButton m_spinHours;
  Gtk::Label      m_labelMins;
  Gtk::SpinButton m_spinMins;
  Gtk::Label      m_labelSecs;
  Gtk::SpinButton m_spinSecs;
  Gtk::Label      m_labelMSecs;
  Gtk::SpinButton m_spinMSecs;
};



/*
 *
 */
class DialogFind : public Gtk::Dialog
{
public:
  DialogFind(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
	~DialogFind();

	void init(Document *doc);

	void findNext();

	/*
	 *	iter = pointe sur la ligne trouver
	 *	fstart = position du mots dans le texte s'il est trouver
	 */
	bool findNext(Gtk::TreeIter &iter, Glib::ustring::size_type &fstart);

protected:
  Glib::ustring getFindText();
	bool getMatchCase();
	bool getMatchEntireWordOnly();

protected:
	Gtk::Entry* m_entryText;
	Gtk::CheckButton* m_checkMatchCase;
	Gtk::CheckButton* m_checkMatchEntireWordOnly;
	bool restart;
};



/*
 *
 */
class DialogFindAndReplace : public Gtk::Dialog
{
public:
  DialogFindAndReplace(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

  Glib::ustring getFindText();
  Glib::ustring getReplaceText();

  enum RESPONSE
  {
    CLOSE=0,
    REPLACE_ALL=1,
    REPLACE=2,
    FIND=3
  };
protected:
  Gtk::Entry*  m_entryFind;

  Gtk::Entry*  m_entryReplace;
};




/*
 *
 */
class DialogAbout : public Gtk::AboutDialog
{
public:
  DialogAbout(Gtk::Window &win);
protected:
  Gtk::Label  m_label;
};

/*
class DialogOpen : public Gtk::FileChooserDialog
{
public:
  DialogOpen(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
};
*/

/*
 *
 */
class DialogOpenSubtitle : public DialogFileChooser
{
public:
  DialogOpenSubtitle();
};



/*
 *
 */
class DialogSaveSubtitle : public DialogFileChooser
{
public:
  DialogSaveSubtitle();
};



/*
 *
 */
class DialogOpenMovie : public Gtk::FileChooserDialog
{
public:
  DialogOpenMovie();
};



/*
 *
 */
class DialogImportText : public DialogFileChooser
{
public:
  DialogImportText();
};



/*
 *
 */
class DialogExportText : public DialogFileChooser
{
public:
  DialogExportText();
};




/*
 *
 */
class DialogScale : public Gtk::Dialog
{
public:
  DialogScale(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

	void init(Document *doc);
protected:
	void on_spin1_changed();
	void on_spin2_changed();

	void on_entry1_changed();
	void on_entry2_changed();
protected:
	Glib::RefPtr<SubtitleModel> m_model;

	Gtk::SpinButton*	m_spinSource1;
	Gtk::Entry*				m_entryDest1;

	Gtk::SpinButton*	m_spinSource2;
	Gtk::Entry*				m_entryDest2;
};

/*
 *
 */
class DialogMoveSubtitle : public Gtk::Dialog
{
public:
  DialogMoveSubtitle();
  DialogMoveSubtitle(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

  void set_value(const SubtitleTime &start);
  SubtitleTime get_value();

protected:
  Gtk::Table	  m_table;
  Gtk::Label	  m_labelDest;
  GtkSubtitleTime m_timer;
  Gtk::Label	  m_labelOption;
  Gtk::RadioButton  m_radioAll;
  Gtk::RadioButton  m_radioCurrent;
};

/*
 *
 */
class DialogChangeFPS : public Gtk::Dialog
{
public:
  DialogChangeFPS(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

	void init(Document *doc);
protected:
  double  get_source();
  double  get_dest();
protected:

  Gtk::SpinButton* m_spinSrc;
  Gtk::SpinButton* m_spinDest;

	Gtk::ComboBox* m_comboSrc;
	Gtk::ComboBox* m_comboDest;
};

/*
 *
 */
class DialogSetAllEndTime : public Gtk::Dialog
{
public:
  DialogSetAllEndTime();

  enum Type
  {
    ADD,
    SET,
  };

  Type		get_type();
  SubtitleTime	get_time();
  
protected:
  Gtk::Table	  m_table;

  Gtk::Label	  m_labelRadio;
  Gtk::RadioButton  m_radioAdd;
  Gtk::RadioButton  m_radioSet;

  Gtk::Label	  m_label;
  GtkSubtitleTime m_time;
};


/*
 *
 */
class DialogStyleEditor : public Gtk::Dialog
{
public:
  DialogStyleEditor(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
//	DialogStyleEditor(Document* document);
	void init(Document *doc);
protected:
	/*
	 *	y a t'il un style select pour edition
	 */
	bool has_select();
	
	void on_select_style_changed();

	/*
	 *	init l'interface avec le style select
	 */
	void on_init_style();
	
	void on_entry_name_changed();

	void on_button_font_changed();
	
	void on_button_bold_toggled();
	void on_button_italic_toggled();
	void on_button_underline_toggled();
	void on_button_strikeout_toggled();
	
	void on_color_primary_changed();
	void on_color_secondary_changed();
	void on_color_outline_changed();
	void on_color_shadow_changed();
	
	void on_spin_margin_l_changed();
	void on_spin_margin_r_changed();
	void on_spin_margin_v_changed();

	void on_spin_angle_changed();
	void on_spin_scale_x_changed();
	void on_spin_scale_y_changed();
	void on_spin_spacing_changed();

	void on_radio_outline_changed();
	void on_spin_outline_changed();
	void on_spin_shadow_changed();
	void on_radio_opaque_box_changed();

	void on_button_alignment_changed(unsigned int num);

	void on_new_style();
	void on_delete_style();
	void on_copy_style();
	void on_manage_styles();
protected:
	Document*	ref_document;
	Glib::RefPtr<StyleModel> ref_styleModel;

	StyleColumnRecorder column;
	Gtk::TreeIter								iter;
	
	Gtk::Button*				buttonNewStyle;
	Gtk::Button*				buttonDeleteStyle;
	Gtk::Button*				buttonCopyStyle;
	Gtk::Button*				buttonManageStyles;
	
	Gtk::TreeView*			treeviewStyle;


	Gtk::Widget*				vbox2;	// box widget edit
	Gtk::Entry*					entryName;
	Gtk::FontButton*		buttonFont;

	Gtk::ToggleButton*	buttonBold;
	Gtk::ToggleButton*	buttonItalic;
	Gtk::ToggleButton*	buttonUnderline;
	Gtk::ToggleButton*	buttonStrikeout;

	Gtk::ColorButton*		buttonPrimaryColor;
	Gtk::ColorButton*		buttonSecondaryColor;
	Gtk::ColorButton*		buttonOutlineColor;
	Gtk::ColorButton*		buttonShadowColor;
	
	Gtk::SpinButton*		spinMarginL;
	Gtk::SpinButton*		spinMarginR;
	Gtk::SpinButton*		spinMarginV;

	Gtk::SpinButton*		spinAngle;
	Gtk::SpinButton*		spinScaleX;
	Gtk::SpinButton*		spinScaleY;
	Gtk::SpinButton*		spinSpacing;

	Gtk::RadioButton*		radioOutline;
	Gtk::SpinButton*		spinOutline;
	Gtk::SpinButton*		spinShadow;
	Gtk::RadioButton*		radioOpaqueBox;

	Gtk::ToggleButton*	buttonAlignment[9];
};



/*
 *
 */
class DialogScriptProperties : public Gtk::Dialog
{
public:
	DialogScriptProperties(Document *doc);

protected:
	void init(ScriptInfo *info);

	void on_entry_script_type_changed();
	void on_entry_point_time_changed();
	
	void on_entry_title_changed();
	void on_entry_original_script_changed();
	void on_entry_original_translation_changed();
	void on_entry_original_editing_changed();
	void on_entry_original_timing_changed();
	void on_entry_sync_point_changed();
	void on_entry_script_updated_by_changed();
	void on_entry_update_details_changed();

	void on_entry_play_res_x_changed();
	void on_entry_play_res_y_changed();
	void on_entry_play_depth_changed();
	
	void on_combo_collisions_changed();
	void on_entry_warp_style_changed();

	void on_spin_timer_changed();
protected:
	ScriptInfo *scriptInfo;

	Gtk::Entry*	entryScriptType;
	Gtk::Entry*	entryPointTime;
	
	Gtk::Entry*	entryTitle;
	Gtk::Entry*	entryOriginalScript;
	Gtk::Entry*	entryOriginalTranslation;
	Gtk::Entry*	entryOriginalEditing;
	Gtk::Entry*	entryOriginalTiming;
	Gtk::Entry*	entrySyncPoint;
	Gtk::Entry*	entryScriptUpdatedBy;
	Gtk::Entry*	entryUpdateDetails;
	
	Gtk::Entry*	entryPlayResX;
	Gtk::Entry*	entryPlayResY;
	Gtk::Entry*	entryPlayDepth;
	
	Gtk::ComboBoxEntry*	comboCollisions;
	Gtk::Entry*	entryWarpStyle;

	Gtk::SpinButton*	spinTimer;
};


/*
 *
 */
class DialogPreference : public Gtk::Dialog
{
public:
	DialogPreference(SubtitleView *view);
};


#endif//_Dialog_h

