/*
 *  subtitle editor
 *	http://kitone.free.fr/subtitleeditor/
 *
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */


#include "Document.h"
#include "SubtitleSSA.h"
#include "SubtitleASS.h"
#include "SubtitleSubRip.h"
#include "SubtitleMicroDVD.h"


/*
 *
 */
Document::Document()
{
	m_charset = "UTF-8";

	m_subtitleModel = Glib::RefPtr<SubtitleModel>(new SubtitleModel);

	m_styleModel = Glib::RefPtr<StyleModel>(new StyleModel);

	//m_nameModel = Glib::RefPtr<NameModel>(new NameModel);
}


/*
 *
 */
Document::~Document()
{
}

/*
 *	utilise simplement l'extention pour determiner quel type de fichier
 *	"srt" "ass" ou "ssa"
 *	en cas d'erreur, retourne ssa
 */
Glib::ustring Document::getFormat()
{
	if(filenameDocument.find(".ssa") != Glib::ustring::npos)
		return "ssa";
	else if(filenameDocument.find(".ass") != Glib::ustring::npos)
		return "ass";	
	else if(filenameDocument.find(".srt") != Glib::ustring::npos)
		return "srt";
	else if(filenameDocument.find(".sub") != Glib::ustring::npos)
		return "sub";
	
	return "ssa";
}

/*
 *	retourne le charset (encodage) du document
 *	a permet de sauvegarder dans le meme encodage qu'a la lecture
 *	...
 */
Glib::ustring Document::getCharset()
{
	return m_charset;
}
	
/*
 *
 */
void Document::clear()
{
	m_scriptInfo = ScriptInfo();
	
	filenameDocument.clear();
	filenameMovie.clear();
	
	m_subtitleModel->clear();
	m_styleModel->clear();
	//m_nameModel->clear();
}

/*
 *
 */
bool Document::open(const Glib::ustring &filename, const Glib::ustring &encoding)
{
	clear();
	
	if(filename.find(".ssa") != Glib::ustring::npos)
	{
		SubtitleSSA ssa(this);
		if(ssa.open(filename, encoding))
		{
			m_charset = ssa.get_charset();

			filenameDocument = filename;
			return true;
		}
	}
	else if(filename.find(".ass") != Glib::ustring::npos)
	{
		try
		{
			SubtitleASS ass(this);
			if(ass.open(filename, encoding))
			{
				m_charset = ass.get_charset();

				filenameDocument = filename;
				return true;
			}
		}catch(Glib::Error &ex)
		{
			std::cerr << ex.what() << std::endl;
		}
	}
	else if(filename.find(".srt") != Glib::ustring::npos)
	{
		SubtitleSubRip srt(this);
		if(srt.open(filename,encoding))
		{
			m_charset = srt.get_charset();
			
			filenameDocument = filename;
			return true;
		}
	}
	else if(filename.find(".sub") != Glib::ustring::npos)
	{
		SubtitleMicroDVD mdvd(this);
		if(mdvd.open(filename, encoding))
		{
			m_charset = mdvd.get_charset();

			filenameDocument = filename;
			return true;
		}
	}
	else
	{
		std::cerr << "Document::open > failed" << std::endl;
	}
	return false;
}

/*
 *
 */
bool Document::save(const Glib::ustring &filename, const Glib::ustring &encoding, bool rename)
{
	Glib::ustring charset=encoding;

	//
	if(!charset.empty() && rename)
		m_charset = charset;
	//
	if(encoding.empty())
		charset = getCharset();

	
	if(rename)
		filenameDocument = filename;

	if(filename.find(".ssa") != Glib::ustring::npos)
	{
		SubtitleSSA ssa(this);
		return ssa.save(filename, charset);
	}
	else if(filename.find(".ass") != Glib::ustring::npos)
	{
		SubtitleASS ass(this);
		return ass.save(filename, charset);
	}
	else if(filename.find(".srt") != Glib::ustring::npos)
	{
		SubtitleSubRip srt(this);
		return srt.save(filename,charset);
	}
	else if(filename.find(".sub") != Glib::ustring::npos)
	{
		SubtitleMicroDVD sub(this);
		return sub.save(filename, charset);
	}
	return false;
}
