#include "MenuBar.h"
#include "utility.h"
#include <iostream>
#include "SubtitleEditor.h"


MenuBar::MenuBar(Gtk::Window &window)
:Gtk::VBox(false, 0)
{
	SubtitleEditor *instance = SubtitleEditor::getInstance();
	
	m_refActionGroup = Gtk::ActionGroup::create();

#define SIGNAL(callback) sigc::mem_fun(instance, &SubtitleEditor::callback)
#define SIGNAL_THIS(callback) sigc::mem_fun(this, &MenuBar::callback)

	// menu-file
	m_refActionGroup->add(Gtk::Action::create("menu-file", _("_File")));
	m_refActionGroup->add(Gtk::Action::create("new-subtitle", Gtk::Stock::NEW), SIGNAL(on_new_subtitle));
	m_refActionGroup->add(Gtk::Action::create("open-subtitle", Gtk::Stock::OPEN, _("_Open Subtitle")), SIGNAL(on_open_subtitle));
	m_refActionGroup->add(Gtk::Action::create("save-subtitle", Gtk::Stock::SAVE, _("_Save Subtitle")), SIGNAL(on_save_subtitle));
	m_refActionGroup->add(Gtk::Action::create("save-as-subtitle", Gtk::Stock::SAVE_AS, _("Save As Subtitle")), SIGNAL(on_save_as_subtitle));
	m_refActionGroup->add(Gtk::Action::create("properties", _("Properties")), SIGNAL(on_properties));
	m_refActionGroup->add(Gtk::Action::create("import-text", _("Import Text")), SIGNAL(on_import_text));
	m_refActionGroup->add(Gtk::Action::create("export-text", _("Export Text")), SIGNAL(on_export_text));
	m_refActionGroup->add(Gtk::Action::create("quit", Gtk::Stock::QUIT), SIGNAL(on_quit));
	
	// menu-option
	m_refActionGroup->add(Gtk::Action::create("menu-option", _("_Options")));
	m_refActionGroup->add(Gtk::Action::create("preferences", Gtk::Stock::PREFERENCES), SIGNAL(on_preferences));

	
	// menu-view
	m_refActionGroup->add(Gtk::Action::create("menu-view", _("_View")));
	
	m_refActionGroup->add(Gtk::Action::create("setup-view-simple", _("Simple")), SIGNAL_THIS(on_setup_view_simple));
	m_refActionGroup->add(Gtk::Action::create("setup-view-advanced", _("Advanced")), SIGNAL_THIS(on_setup_view_advanced));
	m_refActionGroup->add(Gtk::Action::create("setup-view-translation", _("Translation")), SIGNAL_THIS(on_setup_view_translation));

	// menu-columns
	m_refActionGroup->add(Gtk::Action::create("menu-columns", _("Columns")));

#define SIGNAL_COLUMN(column) sigc::bind<Glib::ustring>(sigc::mem_fun(this, &MenuBar::on_toggle_column), column)
	
	m_refActionGroup->add(Gtk::ToggleAction::create("number", _("Number")), SIGNAL_COLUMN("number"));
	m_refActionGroup->add(Gtk::ToggleAction::create("layer", _("Layer")), SIGNAL_COLUMN("layer"));
	m_refActionGroup->add(Gtk::ToggleAction::create("start", _("Start")), SIGNAL_COLUMN("start"));
	m_refActionGroup->add(Gtk::ToggleAction::create("end", _("End")), SIGNAL_COLUMN("end"));
	m_refActionGroup->add(Gtk::ToggleAction::create("style", _("Style")), SIGNAL_COLUMN("style"));
	m_refActionGroup->add(Gtk::ToggleAction::create("name", _("Name")), SIGNAL_COLUMN("name"));
	m_refActionGroup->add(Gtk::ToggleAction::create("margin-r", _("Margin Left")), SIGNAL_COLUMN("margin-r"));
	m_refActionGroup->add(Gtk::ToggleAction::create("margin-l", _("Margin Right")), SIGNAL_COLUMN("margin-l"));
	m_refActionGroup->add(Gtk::ToggleAction::create("margin-v", _("Margin Vertical")), SIGNAL_COLUMN("margin-v"));
	m_refActionGroup->add(Gtk::ToggleAction::create("effect", _("Effect")), SIGNAL_COLUMN("effect"));
	m_refActionGroup->add(Gtk::ToggleAction::create("text", _("Text")), SIGNAL_COLUMN("text"));
	m_refActionGroup->add(Gtk::ToggleAction::create("translation", _("Translation")), SIGNAL_COLUMN("translation"));

#undef SIGNAL_COLUMN

	// menu-edit
	m_refActionGroup->add(Gtk::Action::create("menu-edit", _("_Edit")));
	m_refActionGroup->add(Gtk::Action::create("move-subtitle", Gtk::Stock::JUMP_TO, _("Move Subtitle_")),
			Gtk::AccelKey(GDK_Return, Gdk::MOD1_MASK), SIGNAL(on_move_subtitle));
	m_refActionGroup->add(Gtk::Action::create("delete-subtitle", Gtk::Stock::DELETE),
    Gtk::AccelKey(GDK_Delete, Gdk::CONTROL_MASK), SIGNAL(on_delete_subtitle));
	m_refActionGroup->add(Gtk::Action::create("insert-before", Gtk::Stock::GOTO_TOP, _("Insert Before")),
    Gtk::AccelKey(GDK_Insert, Gdk::CONTROL_MASK), SIGNAL(on_insert_before_subtitle)); 
	m_refActionGroup->add(Gtk::Action::create("insert-after", Gtk::Stock::GOTO_BOTTOM, _("Insert After")),
    Gtk::AccelKey(GDK_Insert, (Gdk::ModifierType)0), SIGNAL(on_insert_after_subtitle));
	m_refActionGroup->add(Gtk::Action::create("change-fps", Gtk::Stock::CONVERT, _("Change FPS")), SIGNAL(on_change_fps));
	m_refActionGroup->add(Gtk::Action::create("scale", Gtk::Stock::CONVERT, _("Scale")), SIGNAL(on_scale));

	// menu-tools
	m_refActionGroup->add(Gtk::Action::create("menu-tools", _("_Tools")));
	
	m_refActionGroup->add(Gtk::Action::create("find", Gtk::Stock::FIND, _("Find")),
    Gtk::AccelKey('f', Gdk::CONTROL_MASK), SIGNAL(on_find));
	m_refActionGroup->add(Gtk::Action::create("find-next", Gtk::Stock::FIND, _("Find Next")),
    Gtk::AccelKey('g', Gdk::CONTROL_MASK), SIGNAL(on_find_next));
	m_refActionGroup->add(Gtk::Action::create("find-and-replace", Gtk::Stock::FIND_AND_REPLACE, _("Find And Replace")),
    Gtk::AccelKey('g', Gdk::SHIFT_MASK | Gdk::CONTROL_MASK), SIGNAL(on_find_and_replace));
	
	m_refActionGroup->add(Gtk::Action::create("remove-line-empty", _("Remove Line Empty")), SIGNAL(on_remove_line_nul));

	m_refActionGroup->add(Gtk::Action::create("check-time", Gtk::Stock::YES, _("Check Time")), SIGNAL(on_check_time));
	m_refActionGroup->add(Gtk::Action::create("spell-check", Gtk::Stock::SPELL_CHECK, _("Spell Check")),
    Gtk::AccelKey('c', Gdk::MOD1_MASK), SIGNAL(on_spell_check)); 
	m_refActionGroup->add(Gtk::Action::create("check-error-encoding", Gtk::Stock::YES, _("Check Error Encoding")), SIGNAL(on_check_error_encoding));
	
	m_refActionGroup->add(Gtk::Action::create("set-all-end-time", _("Set All End Time")), SIGNAL(on_set_all_end_time));
	m_refActionGroup->add(Gtk::Action::create("style-editor", _("Style Editor")), SIGNAL(on_style_editor));

	m_refActionGroup->add(Gtk::Action::create("apply-translation", _("Apply translation")), SIGNAL(on_apply_translation));

	// menu-video
	m_refActionGroup->add(Gtk::Action::create("menu-video", _("_Video")));
	m_refActionGroup->add(Gtk::Action::create("open-movie", Gtk::Stock::OPEN, _("Open Movie")),
    Gtk::AccelKey('o', Gdk::SHIFT_MASK | Gdk::CONTROL_MASK), SIGNAL(on_open_movie));
	m_refActionGroup->add(Gtk::Action::create("play-movie", Gtk::Stock::MEDIA_PLAY, _("_Play Movie")),
    Gtk::AccelKey(GDK_space, Gdk::CONTROL_MASK), SIGNAL(on_play_movie));
	
	// menu-help
	m_refActionGroup->add(Gtk::Action::create("menu-help", _("_Help")));
	m_refActionGroup->add(Gtk::Action::create("about", Gtk::Stock::HELP, _("_About")), SIGNAL(on_about));

	// UIMANAGER

	m_refUIManager = Gtk::UIManager::create();
	m_refUIManager->insert_action_group(m_refActionGroup);

	window.add_accel_group(m_refUIManager->get_accel_group());
	
	m_refUIManager->add_ui_from_file(get_share_dir("menubar.xml"));

	pack_start(*m_refUIManager->get_widget("/menubar"), false,false);

	show_all();

	init_columns_by_subtitleview();

	// disable preferences
	Glib::RefPtr<Gtk::ToggleAction> action = 
		Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action("preferences"));
	if(action)
		action->set_sensitive(false);
	
#undef SIGNAL
#undef SIGNAL_THIS
}

/*
 *	change l'etat d'affichage d'une colonne
 */
void MenuBar::set_toggle_column(const Glib::ustring &column, const bool &state)
{
	try
	{
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(column));

		if(action)
		{
			action->set_active(state);
			SubtitleEditor::getInstance()->getSubtitleView()->set_column_visible(column, state);
		}
	}
	catch(Glib::Error &ex)
	{
		std::cerr << ex.what() << std::endl;
	}
}

/*
 *
 */
void MenuBar::on_toggle_column(const Glib::ustring &column)
{
	Glib::RefPtr<Gtk::ToggleAction> action = 
		Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(column));

	if(action)
	{
		bool state = action->get_active();
		SubtitleEditor::getInstance()->getSubtitleView()->set_column_visible(column, state);
	}
}

	
/*
 *
 */
void MenuBar::init_columns_by_subtitleview()
{
	gchar* columns[]={
		"number", "layer", "start", "end", "style", "name", 
		"margin-r", "margin-l", "margin-v", "effect", 
		"text", "translation"};

	for(unsigned int i=0; i<12; ++i)
	{
		bool state = SubtitleEditor::getInstance()->getSubtitleView()->get_column_visible(columns[i]);
		
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(columns[i]));

		if(action)
		{
			action->set_active(state);
		}
	}
}

/*
 *
 */
void MenuBar::on_setup_view_simple()
{
	set_toggle_column("number", true);
	set_toggle_column("layer", false);
	set_toggle_column("start", true);
	set_toggle_column("end", true);
	set_toggle_column("style", false);
	set_toggle_column("name", false);
	set_toggle_column("margin-r", false);
	set_toggle_column("margin-l", false);
	set_toggle_column("margin-v", false);
	set_toggle_column("effect", false);
	set_toggle_column("text", true);
	set_toggle_column("translation", false);
}

void MenuBar::on_setup_view_advanced()
{
	set_toggle_column("number", true);
	set_toggle_column("layer", true);
	set_toggle_column("start", true);
	set_toggle_column("end", true);
	set_toggle_column("style", true);
	set_toggle_column("name", true);
	set_toggle_column("margin-r", false);
	set_toggle_column("margin-l", false);
	set_toggle_column("margin-v", false);
	set_toggle_column("effect", false);
	set_toggle_column("text", true);
	set_toggle_column("translation", false);
}

void MenuBar::on_setup_view_translation()
{
	set_toggle_column("number", true);
	set_toggle_column("layer", false);
	set_toggle_column("start", false);
	set_toggle_column("end", false);
	set_toggle_column("style", false);
	set_toggle_column("name", false);
	set_toggle_column("margin-r", false);
	set_toggle_column("margin-l", false);
	set_toggle_column("margin-v", false);
	set_toggle_column("effect", false);
	set_toggle_column("text", true);
	set_toggle_column("translation", true);
}
