/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */


#include "SubtitleFormat.h"
#include "Encodings.h"
#include <iostream>

/*
 *
 */
void SubtitleFormat::set_charset(const std::string &charset)
{
	m_charset = charset;
}

/*
 *
 */
std::string SubtitleFormat::get_charset()
{
	return m_charset;
}

	
/*
 *
 */
bool SubtitleFormat::open(const Glib::ustring &filename, const Glib::ustring &encoding)
{
	set_charset(encoding);
	
	//debug_msg("SubtitleFormat::open > " + filename + " (" + encoding + ")");
	debug_msg("SubtitleFormat open > %s (%s)", filename.c_str(), encoding.c_str());

	return false;
}
	
/*
 *
 */
bool SubtitleFormat::save(const Glib::ustring &filename, const Glib::ustring &encoding)
{
	set_charset(encoding);

	//debug_msg("SubtitleFormat::save > " + filename.c_str() + " (" + encoding.c_str() + ")");
	debug_msg("SubtitleFormat save > %s (%s)", filename.c_str(), encoding.c_str());

	return false;
}

/*
 *
 */
std::string convert_to_utf8_from_charset(const std::string &content, const std::string &charset)
{
	try
	{
		if(charset == "UTF-8")
		{
			if(Glib::ustring(content).validate())
			{
				return content;
			}
			else
				std::cerr << "invalid byte sequence" << std::endl;

			return "";
		}

		std::string utf8_content = Glib::convert(content, "UTF-8", charset);
		
		if(Glib::ustring(utf8_content).validate() == false)
			return "";

		return utf8_content;
	}
	catch(Glib::ConvertError& ex)
	{
		std::cerr << "convert_to_utf8_from_charset(" << charset << ") error" << std::endl;
		std::cerr << ex.what() << std::endl;
	}
	
	return "";
}

/*
 *
 */
std::string convert_to_utf8(const std::string &content, std::string& encoding)
{
	if(encoding.empty() == false)
	{
		return convert_to_utf8_from_charset(content, encoding);
	}
	else
	{
		// Automatically detect the encoding used
		for(unsigned int i=0; encodings_info[i].name!= NULL; ++i)
		{
			std::string charset = encodings_info[i].charset;
		
			std::string utf8_content = convert_to_utf8_from_charset(content,charset);
		
			if(utf8_content.empty() == false)
			{
				encoding = charset;
				//
				//std::cout << charset << ":" << content << std::endl;
				return utf8_content;
			}
		}
	}


	std::cerr << "SubtitleEditor has not been able to automatically determine the encoding of the file you want to open.";
	return "Invalid Encoding";
}

/*
 *	convertir text en utf8
 *	le charset du text est defini par m_charset
 */
Glib::ustring SubtitleFormat::charset_to_utf8(const std::string &content)
{
	try
	{
		return convert_to_utf8(content, m_charset);
	}
	catch(Glib::ConvertError& ex)
	{
		std::cerr << m_charset << " : " << ex.what() << std::endl;
	}

	return "";
}

/*
 *	convertir le text (utf-8) en codage defini par m_charset
 */
std::string SubtitleFormat::utf8_to_charset(const Glib::ustring &content)
{
	try
	{
		if(m_charset.empty())
		{
			std::cerr << "utf8_to_charset charset is empty" << std::endl;
			return content;
		}
		
		std::string res = Glib::convert(content, m_charset, "UTF-8");
		return res;
	}
	catch(Glib::ConvertError& ex)
	{
		std::cerr << content << std::endl;
		std::cerr << "utf8_to_charset[" << m_charset << "] " << ex.what() << std::endl;
		std::cerr << std::endl;
	}

	return "";
}
